import { Component, EventEmitter, Input, Output } from '@angular/core';
import { EmployeeNameAndSkillDataDTO, EmployeeRequestPutDTO, EmployeeResponseDTO } from '../../models/mitarbeiter';
import { AbstractControl, FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { HttpClient } from '@angular/common/http';
import { Router } from '@angular/router';
import { AsyncPipe, NgFor } from '@angular/common';
import { SkillService } from '../../service/skill.service';
import { QualificationGetDTO } from '../../models/skill';
import { Observable, of } from 'rxjs';
import { EmployeeService } from '../../service/employee.service';

@Component({
  selector: 'app-mitarbeiter-form',
  standalone: true,
  imports: [MitarbeiterFormComponent, ReactiveFormsModule, NgFor, AsyncPipe],
  templateUrl: './mitarbeiter-form.component.html',
  styleUrl: './mitarbeiter-form.component.css'
})
export class MitarbeiterFormComponent {
  @Input() mitarbeiter!: EmployeeResponseDTO;
  @Output() mitarbeiterChange = new EventEmitter<EmployeeResponseDTO>();

  public mitarbeiterForm!: FormGroup;
  public allSkills: Observable<Array<QualificationGetDTO>> = of([]);

  constructor(public http: HttpClient, public router: Router, private employeeService: EmployeeService, private skillService: SkillService) {

  }

  private setupForm() {
    this.mitarbeiterForm = new FormGroup({
      lastName: new FormControl(this.mitarbeiter.lastName),
      firstName: new FormControl(this.mitarbeiter.firstName),
      street: new FormControl(this.mitarbeiter.street),
      postcode: new FormControl(this.mitarbeiter.postcode),
      city: new FormControl(this.mitarbeiter.city),
      phone: new FormControl(this.mitarbeiter.phone),
      newSkill: new FormControl(),
    });
  }

  ngOnChanges(): void {
    this.setupForm();
  }

  ngOnInit(): void {
    this.allSkills = this.skillService.getAllSkills();
    this.setupForm();
  }

  removeSkill(id?: number) {
    this.mitarbeiter.skillSet = this.mitarbeiter.skillSet?.filter(skill => skill.id !== id);
  }

  addSkill() {
    const id = Number(this.mitarbeiterForm.get("newSkill")?.value);
    this.employeeService.addSkillToEmployee(id, this.mitarbeiter);
  }

  submit() {
    this.mitarbeiter.firstName = this.mitarbeiterForm.get("firstName")?.value;
    this.mitarbeiter.lastName = this.mitarbeiterForm.get("lastName")?.value;
    this.mitarbeiter.street = this.mitarbeiterForm.get("street")?.value;
    this.mitarbeiter.postcode = this.mitarbeiterForm.get("postcode")?.value;
    this.mitarbeiter.city = this.mitarbeiterForm.get("city")?.value;
    this.mitarbeiter.phone = this.mitarbeiterForm.get("phone")?.value;

    console.log(this.mitarbeiterForm);
    console.log(this.mitarbeiter);
    this.mitarbeiterChange.emit(this.mitarbeiter);
  }
}
